library bloc_stream;

import 'dart:async';

import 'package:meta/meta.dart';
import 'package:rxdart/rxdart.dart';

abstract class BlocStream<State> extends Stream<State> {
  BlocStream() {
    _subject = BehaviorSubject.seeded(initialValue);
  }

  State get initialValue => null;

  BehaviorSubject<State> _subject;
  State get value => _subject.value;

  @override
  bool get isBroadcast => _subject.isBroadcast;

  @override
  StreamSubscription<State> listen(
    void Function(State) onData, {
    Function onError,
    void Function() onDone,
    bool cancelOnError,
  }) {
    return _subject.listen(
      onData,
      onError: onError,
      onDone: onDone,
      cancelOnError: cancelOnError,
    );
  }

  // Methods for adding data to the _subject
  @protected
  void add(State value) {
    if (_subject.isClosed) return;
    _subject.add(value);
  }

  @protected
  void addError(Object error, [StackTrace stackTrace]) {
    if (_subject.isClosed) return;
    _subject.addError(error, stackTrace);
  }

  // Helper method to reduce boilerplate when composing BLoC's together.
  final _closers = <void Function()>[];
  @protected
  void cancelOnClose(StreamSubscription subscription,
      [void Function() onClose]) {
    _closers.add(() {
      subscription.cancel();
      if (onClose != null) onClose();
    });
  }

  @mustCallSuper
  Future<void> close() {
    _closers.forEach((fn) => fn());
    return _subject.close();
  }
}
